
class Constants():
    ID = "id"
    STATE = "state"
    ACTIVE = "Active"
    NAME = "name"
    LATITUDE = "latitude"
    LONGITUDE = "longitude"
    ADDRESS = "address"
    ZIP = "zip"
    CITY = 'city'
    VEHICLES = "vehicles"
    TYPE = "type"
    BATTERY_LEVEL = "ebike_battery_level"
    BIKE = "Bike"
    EBIKE = "E-Bike"



from pypublibike.location import Location
from pypublibike.bike import Bike, Ebike
from pypublibike.constants import Constants
import requests
from typing import Optional, List


class Station:

    def __init__(self, stationId: int, location: Location = None):
        self._stationId = stationId
        self._stationActive = None
        self._name = None
        self._location = None
        self._address = None
        self._zip = None
        self._city = None
        self._bikes: List[Bike] = []
        self._ebikes: List[Ebike] = []
        if location is not None:
            self._location = location

    def _retrieveStation(self, _stationId: int):
        return requests.get(f'https://api.publibike.ch/v1/public/stations/{_stationId}').json()

    def refresh(self):
        response = self._retrieveStation(self._stationId)
        self._active = Constants.ACTIVE == response[Constants.STATE][Constants.NAME]
        self._name = response[Constants.NAME]
        self._location = Location(
            float(response[Constants.LATITUDE]), float(response[Constants.LONGITUDE]))
        self._address = response[Constants.ADDRESS]
        self._zip = int(response[Constants.ZIP])
        self._city = response[Constants.CITY]
        vehicles = response[Constants.VEHICLES]

        self._ebikes.clear()
        for entry in filter(lambda vehicle: vehicle[Constants.TYPE][Constants.NAME] == Constants.EBIKE, vehicles):
            batteryLevel = 0
            if None is not entry[Constants.BATTERY_LEVEL]:
                batteryLevel = int(entry[Constants.BATTERY_LEVEL])
            self._ebikes.append(
                Ebike(int(entry[Constants.NAME]), batteryLevel))

        self._bikes.clear()
        for entry in filter(lambda vehicle: vehicle[Constants.TYPE][Constants.NAME] == Constants.BIKE, vehicles):
            self._bikes.append(Bike(int(entry[Constants.NAME])))

    @property
    def stationId(self) -> int:
        return self._stationId

    @property
    def active(self) -> Optional[bool]:
        return self._active

    @property
    def name(self) -> Optional[str]:
        return self._name

    @property
    def location(self) -> Optional[Location]:
        return self._location

    @property
    def address(self) -> Optional[str]:
        return self._address

    @property
    def zip(self) -> Optional[int]:
        return self._zip

    @property
    def city(self) -> Optional[str]:
        return self._city

    @property
    def bikes(self) -> List[Bike]:
        return self._bikes

    @property
    def ebikes(self) -> List[Ebike]:
        return self._ebikes



from pypublibike.station import Station
from pypublibike.location import Location
from pypublibike.constants import Constants
from haversine import haversine
import requests


class PubliBike:

    def findNearestStationTo(self, location: Location):
        stations = self.getStations()
        nearestStation = None
        distanceNearestStation = 100000
        for station in stations:
            distance = haversine(
                (station.location.latitude, station.location.longitude), (location.latitude, location.longitude))
            if distance < distanceNearestStation:
                nearestStation = station
                distanceNearestStation = distance
        return nearestStation

    def getStations(self) -> list:
        stations = []
        r = requests.get("https://api.publibike.ch/v1/public/stations/")
        for station in r.json():
            location = Location(
                float(station[Constants.LATITUDE]), float(station[Constants.LONGITUDE]))
            stations.append(Station(station[Constants.ID], location))
        return stations



import unittest
import os
import json
from pypublibike.station import Station
from pypublibike.location import Location


class TestStation(unittest.TestCase):

    def getSampleJson(self, _unused):
        with open(os.path.join("test", "test_data", "station_happyday.json")) as f:
            sampleJson = f.read()
            return json.loads(sampleJson)

    def test_happyday(self):
        stationId = 123
        station = Station(stationId)
        station._retrieveStation = self.getSampleJson

        station.refresh()
        self.assertEqual(station.name, "Köniz Zentrum")
        self.assertTrue(station.active)
        self.assertEqual(station.address, "Bläuacker 10")
        self.assertEqual(station.zip, 3098)
        self.assertEqual(station.city, "Köniz")
        self.assertEqual(station.bikes[0].name, 102783)
        self.assertEqual(station.ebikes[0].name, 501352)
        self.assertEqual(station.ebikes[0].batteryLevel, 44)
        self.assertEqual(station.location, Location(46.9229774, 7.4141121))
        self.assertEqual(station.stationId, stationId)

    def test_live(self):
        station = Station(421)
        station.refresh()


if __name__ == '__main__':
    unittest.main()



# Always prefer setuptools over distutils
from setuptools import setup, find_packages
from os import path

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='pypublibike',
    version='2.1.0',
    description='A python wrapper around the PubliBike API',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/eliabieri/pypublibike',
    author='Elia Bieri',
    author_email='bieri.elia@gmail.com',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Topic :: Internet',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],

    packages=find_packages(
        exclude=["tests", "*.tests", "*.tests.*", "tests.*"]),
    python_requires='>=3.7',
    install_requires=['requests', 'haversine'],
)
